@@ -55,13 +55,13 @@ isFALSE <- function(x){
 #' @export
 tinytest <- function(result, call
     , diff = NA_character_
-    , short= NA_character_
+    , short= c(NA_character_,"data","attr","xcpt", "envv","wdir","file")
     , info = NA_character_
     , file = NA_character_
     , fst  = NA_integer_
     , lst  = NA_integer_
     ,...){
-
+  short <- match.arg(short)
   structure(result         # logical TRUE/FALSE
     , class    = "tinytest"
     , call     = call  # call creating or motivating the object
@@ -79,7 +79,8 @@ na_str <- function(x) if ( is.na(x) ) "" else as.character(x)
 
 oneline <- function(x) sub("\\n.+","...",x)
 indent <- function(x, with="     "){
-  gsub("\\n *",paste0("\n",with),paste0(with,sub("^ +","",x)))
+  if (is.na(x)) ""
+  else gsub("\\n *",paste0("\n",with),paste0(with,sub("^ +","",x)))
 }
 
 lineformat <- function(x){
@@ -115,17 +116,15 @@ format.tinytest <- function(x,type=c("long","short"), ...){
             else if (isFALSE(x)) sprintf("FAILED[%s]",short)
             else if (is.na(x)  ) sprintf("SIDEFX[%s]",short)
   longfmt <- "----- %s: %s<%s--%s>\n%s"
-  # make room for diff and info fields when necessary
-  if (isFALSE(x)||is.na(x)) longfmt <- paste0(longfmt, "\n%s")
-  if (!is.na(d$info)) longfmt <- paste0(longfmt,"\n%s")
 
   if (type == "short"){
     sprintf("%s: %s<%s--%s> %s", result, basename(file), fst, lst, oneline(call))
   }  else {
-    sprintf(longfmt, result, file, fst, lst
-                , indent(call, with=" call| ")
-                , indent(diff, with=" diff| ")
-                , indent(info, with=" info| "))
+    str <- sprintf(longfmt, result, file, fst, lst
+                , indent(call, with=" call| "))
+    if (isFALSE(x)||is.na(x)) str <- paste0(str, "\n", indent(diff, with=" diff| "))
+    if (!is.na(d$info)) str <- paste0(str, "\n", indent(info, with=" info| "))
+    str
   }
 
 }
@@ -176,6 +175,8 @@ longdiff <- function(current, target, alt){
        if ( all(class(current) %in% c("character","ordered","factor", "POSIXt","POSIXct")) ) 
          sprintf("Expected '%s', got '%s'", target, current)
        else sprintf("Expected %s, got %s", target, current)
+  } else if (isTRUE(alt) && is.environment(current)){
+    "Equal environment objects, but with different memory location"
   } else {
     paste0(" ", alt, collapse="\n")
   }
@@ -200,7 +201,7 @@ shortdiff <- function(current, target, ...){
 #'
 #' @param current \code{[R object or expression]} Outcome or expression under scrutiny.
 #' @param target \code{[R object or expression]} Expected outcome
-#' @param tol \code{[numeric]} Test equality to machine rounding. Passed
+#' @param tolerance \code{[numeric]} Test equality to machine rounding. Passed
 #'     to \code{\link[base]{all.equal} (tolerance)}
 #' @param info \code{[character]} scalar. Optional user-defined message. Must
 #'  be a single character string. Multiline comments may be separated by
@@ -233,12 +234,12 @@ shortdiff <- function(current, target, ...){
 #' expect_equal(2, c(x=2))      # FALSE
 #'
 #' @export
-expect_equal <- function(current, target, tol = sqrt(.Machine$double.eps), info=NA_character_, ...){
+expect_equal <- function(current, target, tolerance = sqrt(.Machine$double.eps), info=NA_character_, ...){
 
-  check <- all.equal(target, current, tol=tol, ...)
+  check <- all.equal(target, current, tolerance=tolerance, ...)
   equal <- isTRUE(check)
   diff  <- if (equal) NA_character_ else longdiff( current, target, check) 
-  short <- if (equal) NA_character_ else shortdiff(current, target, tolerance=tol)
+  short <- if (equal) NA_character_ else shortdiff(current, target, tolerance=tolerance)
 
   tinytest(result = equal, call = sys.call(sys.parent(1)), diff=diff, short=short, info=info)
 }
@@ -250,7 +251,7 @@ expect_equal <- function(current, target, tol = sqrt(.Machine$double.eps), info=
 expect_identical <- function(current, target, info=NA_character_){
   result <- identical(current, target)
   diff <-  if (result) NA_character_
-           else longdiff(current, target, all.equal(target, current))
+           else longdiff(current, target, all.equal(target, current, check.attributes=TRUE))
   short <- if (result) NA_character_
            else shortdiff(current, target, tolerance=0)
   tinytest(result=result, call=sys.call(sys.parent(1)), diff=diff
@@ -266,11 +267,11 @@ expect_identical <- function(current, target, info=NA_character_){
 #'
 #' @rdname expect_equal
 #' @export
-expect_equivalent <- function(current, target, tol = sqrt(.Machine$double.eps)
+expect_equivalent <- function(current, target, tolerance = sqrt(.Machine$double.eps)
                             , info=NA_character_, ...){
   out <- expect_equal(current, target
           , check.attributes=FALSE,use.names=FALSE
-          , tol=tol, info=info, ...)
+          , tolerance=tolerance, info=info, ...)
   attr(out, 'call') <- sys.call(sys.parent(1))
   out
 }
@@ -282,7 +283,8 @@ expect_true <- function(current, info=NA_character_){
   call <- sys.call(sys.parent(1))
   if (!result){
     this <- if ( isFALSE(current) ) "FALSE"
-            else if (is.logical(current)) sprintf("'logical' of length %d",length(current))
+            else if ( length(current) == 1 && is.na(current)) "NA"
+            else if ( is.logical(current)) sprintf("'logical' of length %d",length(current))
             else sprintf("object of class '%s'",class(current))
     diff  <- sprintf("Expected TRUE, got %s", this)
     short <- shortdiff(TRUE, FALSE)
@@ -298,10 +300,11 @@ expect_false <- function(current, info=NA_character_){
   result <- isFALSE(current)
   call   <- sys.call(sys.parent(1))
   if (!result){
-    this <- if ( isFALSE(current) ) "TRUE"
+    this <- if ( isTRUE(current) ) "TRUE"
+            else if (length(current) == 1 && is.na(current)) "NA"
             else if (is.logical(current)) sprintf("'logical' of length %d",length(current))
             else sprintf("object of class '%s'",class(current))
-    diff  <- "Expected FALSE, got TRUE"
+    diff  <- sprintf("Expected FALSE, got %s", this)
     short <- shortdiff(TRUE, FALSE)
     tinytest(result, call=call,diff=diff, short=short, info=info)
   } else {
@@ -326,7 +329,7 @@ expect_false <- function(current, info=NA_character_){
 #' expect_silent(1+1)           # TRUE
 #' expect_silent(1+"a")         # FALSE
 #' expect_silent(print("hihi")) # TRUE, nothing goes to screen
-#' expect_silent(print("hihi", quiet=FALSE)) # FALSE, and printed
+#' expect_silent(print("hihi"), quiet=FALSE) # TRUE, and printed
 #'
 #' @export
 expect_silent <- function(current, quiet=TRUE, info=NA_character_){
@@ -400,18 +403,49 @@ expect_null <- function(current, info=NA_character_){
 
 
 #' @rdname expect_equal
-#' @param pattern \code{[character]} A regular expression to match the message.
+#' 
+#' @param class \code{[character]} A class string.
+#' @details 
+#'  \code{expect_inherits} fails when \code{\link{inherits}(current,class)} returns \code{FALSE} 
 #' @export
-expect_error <- function(current, pattern=".*", info=NA_character_){
+expect_inherits <- function(current, class, info=NA_character_){
+  call <- sys.call(sys.parent(1))
+  res  <- inherits(current, class)
+  if (isTRUE(res)){
+    tinytest(TRUE, call=call, info=info)
+  } else {
+    tinytest(FALSE, call=call, short="attr"
+      , diff = sprintf("Expected object of class %s, got %s"
+          , paste0("<", paste(class,collapse=", "),">")
+          , paste0("<", paste(class(current), collapse=", "),">")
+      , info=info))
+  }
+
+}
+
+
+
+#' @rdname expect_equal
+#' @param pattern \code{[character]} A regular expression to match the message.
+#' @param class \code{[character]} For condition signals (error, warning, message)
+#'        the class from which the condition should inherit.
+#' @export
+expect_error <- function(current, pattern=".*", class="error", info=NA_character_){
   result <- FALSE
   diff <- "No error"
   
   tryCatch(current, error=function(e){
-            if (grepl(pattern, e$message)){
-                result <<- TRUE
-            } else {
+            matches <- grepl(pattern, e$message)
+            isclass <- inherits(e, class)
+
+            if (matches && isclass){
+              result <<- TRUE
+            } else if (!isclass){
+              diff <<- sprintf("Error of class '%s', does not inherit from '%s'"
+                              , paste(class(e), collapse=", "), class)
+            } else if (!matches){
               diff <<- sprintf("The error message:\n '%s'\n does not match pattern '%s'"
-                             , e$message, pattern)
+                              , e$message, pattern)
             }
   })
   tinytest(result, call = sys.call(sys.parent(1))
@@ -420,84 +454,269 @@ expect_error <- function(current, pattern=".*", info=NA_character_){
            , info = info)
 }
 
+
+# helper: format 1st three elements of a list of condition objects
+first_n <- function(L, n=3){
+  i      <- seq_len(min(length(L),n))
+
+
+
+  msgcls <- sapply(L[i], function(m) paste(class(m), collapse=", "))
+   
+  maintype <- sapply(L[i], function(m){
+    if      ( inherits(m, "message") ) "Message"
+    else if ( inherits(m, "warning") ) "Warning"
+    else if ( inherits(m, "error")   ) "Error"
+    else "Condition"
+  }) 
+
+
+   msgtxt <- sub("\\n$","", sapply(L[i], function(m) m$message))
+   
+   out   <- sprintf("%s %d of class <%s>:\n  '%s'",maintype, i, msgcls, msgtxt)
+   paste(out, collapse="\n")
+}
+
+
+
 #' @rdname expect_equal
 #' @export
-expect_warning <- function(current, pattern=".*", info=NA_character_){
-  
-  result <- FALSE
-  diff <- "No warning"
+expect_warning <- function(current, pattern=".*", class="warning", info=NA_character_){
+ 
+  messages <- list()
+  warnings <- list()  
+  errors   <- list()
 
-  withCallingHandlers(current
-    , warning = function(w){
-        if (grepl(pattern, w$message)){
-          result <<- TRUE
-        } else {
-          diff <<- sprintf("The warning message\n '%s'\n does not match pattern '%s'"
-                          , w$message, pattern)
+  tryCatch(withCallingHandlers(current
+      , warning = function(w){ 
+          warnings <<- append(warnings, list(w))
+          invokeRestart("muffleWarning")
         }
-        invokeRestart("muffleWarning")
-      }
+      , message = function(m) {
+          messages <<- append(messages, list(m))
+          invokeRestart("muffleMessage")
+        }
+      )
+    , error  = function(e) errors <<- append(errors, list(e))
   )
 
+  nmsg <- length(messages)
+  nwrn <- length(warnings)
+  nerr <- length(errors)
+ 
+ 
+  results <- sapply(warnings, function(w) {
+    inherits(w, class) && grepl(pattern, w$message)
+  })
+
+  if (any(results)){ ## happy flow
+    result <- TRUE
+    short  <- diff <- NA_character_
+  } else { ## construct diff  message
+    result <- FALSE
+    short  <- "xcpt"
+    diff   <- if ( nwrn == 0 ){
+      "No warning was emitted"
+    } else {
+      n_right_class <- sum(sapply(warnings, function(w) inherits(w, class)))
+      if (n_right_class == 0){
+        head <- sprintf("Found %d warning(s), but not of class '%s'.", nwrn, class)
+        head <- paste(head, "Showing up to three warnings:\n")
+        body <- first_n(warnings)
+        paste(head, body)
+      } else {
+        wrns <- Filter(function(w) inherits(w,class), warnings)
+        head <- sprintf("Found %d warnings(s) of class '%s', but not matching '%s'."
+                      , nwrn, class, pattern)
+        head <- paste(head,"\nShowing up to three warnings:\n")
+        body <- first_n(wrns)
+        paste(head, body) 
+      }
+    }
+  }
+
+  if (!result && (nmsg > 0 || nerr > 0)) 
+    diff <- paste0(diff,sprintf("\nAlso found %d message(s) and %d error(s)"
+              , nmsg, nerr))
+
   tinytest(result, call=sys.call(sys.parent(1))
-           , short = if (result) NA_character_ else "xcpt"
-           , diff  = if (result) NA_character_ else diff
-           , info  = info)
+          , short=short, diff=diff, info=info)
+
 }
 
 
 #' @rdname expect_equal
 #' @export
-expect_message <- function(current, pattern=".*", info=NA_character_){
-  value <- ""
-  tc <- textConnection("value", open="w", local=TRUE)
-  sink(file=tc,type="message", split=FALSE)
-  
+expect_message <- function(current, pattern=".*", class="message", info=NA_character_){
+ 
+  messages <- list()
+  warnings <- list()  
+  errors   <- list()
 
-  result <- TRUE
-  msg    <- ""
-  tryCatch(current
-    , error = function(e){
-        result <<- FALSE
-        msg <<- sprintf("Expected message, got error:\n '%s'",e$message)
-      }
-    , warning = function(w){
-        result <<- FALSE
-        msg <<- paste(sprintf("Expected message, got warning:\n '%s'", w$message)
-                    , collapse="\n")
-      }
+  tryCatch(withCallingHandlers(current
+      , warning = function(w){ 
+          warnings <<- append(warnings, list(w))
+          invokeRestart("muffleWarning")
+        }
+      , message = function(m) {
+          messages <<- append(messages, list(m))
+          invokeRestart("muffleMessage")
+        }
+      )
+    , error  = function(e) errors <<- append(errors, list(e))
   )
-  sink(file = NULL, type="message")
-  close(tc)
-  
-  # collapse the value string in case multiple messages were caught.
-  value <- paste(value, collapse="\n")
-  call <- sys.call(sys.parent(1))
-  # we got a warning or error instead of a message:
-  if (!result){
-    tinytest(
-        result
-      , call
-      , diff  = msg
-      , short = "xcpt" 
-      , info  = info
-    ) 
-  # we got a message, check if it matches 'pattern'
-  } else if (!isTRUE(grepl(pattern, value)) ){
-    df <- if (value == "") "No message"
-          else sprintf("The message\n '%s'\n doen not match pattern '%s'",value,pattern)
-    tinytest(FALSE
-      , call
-      , diff = df
-      , short = "xcpt"
-      , info  = info
-    )
-  } else {
-    tinytest(TRUE, call, info=info)
+
+  nmsg <- length(messages)
+  nwrn <- length(warnings)
+  nerr <- length(errors)
+ 
+ 
+  results <- sapply(messages, function(m) {
+    inherits(m, class) && grepl(pattern, m$message)
+  })
+
+  if (any(results)){ ## happy flow
+    result <- TRUE
+    short <- diff <- NA_character_
+  } else { ## construct diff  message
+    result <- FALSE
+    short <- "xcpt"
+    diff <- if (length(messages) == 0){
+      "No message was emitted"
+    } else {
+      n_right_class <- sum(sapply(messages, function(m) inherits(m, class)))
+      if (n_right_class == 0){
+        head <- sprintf("Found %d message(s), but not of class '%s'.", nmsg, class)
+        head <- paste(head, "Showing up to three messages:\n")
+        body <- first_n(messages)
+        paste(head, body)
+      } else {
+        msgs <- Filter(function(m) inherits(m,class), messages)
+        head <- sprintf("Found %d message(s) of class '%s', but not matching '%s'."
+                      , nmsg, class, pattern)
+        head <- paste(head,"\nShowing up to three messages:\n")
+        body <- first_n(msgs)
+        paste(head, body) 
+      }
+    }
   }
-  
+
+  if (!result && (nwrn > 0 || nerr > 0)) 
+    diff <- paste0(diff,sprintf("\nAlso found %d warning(s) and %d error(s)"
+              , nwrn, nerr))
+
+  tinytest(result, call=sys.call(sys.parent(1))
+          , short=short, diff=diff, info=info)
+
+
 }
 
+#' @rdname expect_equal
+#'
+#' @details
+#'
+#' \code{expect_stdout} Expects that output is written to \code{stdout},
+#' for example using \code{cat} or \code{print}. Use \code{pattern} to
+#' specify a regular expression matching the output.
+#'
+#'
+#' @export
+expect_stdout <- function(current, pattern=".*", info=NA_character_){
+  value <- ""
+  msg <- NA_character_
+  
+  tc <- textConnection("value", open="w", local=TRUE)
+  
+  sink(file=tc, type="output", split=FALSE)
+    tryCatch(current
+      , error=function(e){sink(file=NULL, type="output"); stop(e)}
+    )
+  sink(file = NULL, type="output")
+  close(tc)
+
+  value <- paste(value, collapse="\n")
+  result <- grepl(pattern, value)
+  if (!result)
+    msg <- sprintf("output '%s'\n does not match pattern '%s'", value, pattern)
+
+  tinytest(result, call = sys.call(sys.parent(1))
+           , short= if(result) NA_character_ else "xcpt"
+           , diff = msg
+           , info = info)
+}
+
+
+#' Compare object with object stored in a file
+#'
+#' Compares the current value with a value stored to file with
+#' \code{\link{saveRDS}}.  If the  file does not exist, the current value is
+#' stored into file, and the test returns \code{expect_null(NULL)}.
+#'
+#' @param current \code{[R object or expression]} Outcome or expression under 
+#'        scrutiny.
+#' @param file \code{[character]} File where the \code{target} is stored. If 
+#'        \code{file} does not exist, \code{current} will be stored there.
+#' @param ... passed to \code{\link{expect_equal}}, respectively \code{\link{expect_equivalent}}.
+#'
+#' @note
+#' Be aware that on CRAN it is not allowed to write data to user space. So make
+#' sure that the file is either stored with your tests, or generated with
+#' \code{\link{tempfile}}, or the test is skipped on CRAN, using
+#' \code{\link{at_home}}.
+#' 
+#' Also note that \code{\link{build_install_test}} clones the package and
+#' builds and tests it in a separate R session in the background. This means
+#' that if you create a file located at \code{tempfile()} during the run, this
+#' file is destroyed when the separate R session is closed.
+#'
+#'
+#' @family test-functions
+#'
+#'
+#' @examples
+#' filename <- tempfile()
+#' # this gives TRUE: the file does not exist, but is created now.
+#' expect_equal_to_reference(1, file=filename)
+#' # this gives TRUE: the file now exists, and its contents is equal
+#' # to the current value
+#' expect_equal_to_reference(1, file=filename)
+#' # this gives FALSE: the file exists, but is contents is not equal
+#' # to the current value
+#' expect_equal_to_reference(2, file=filename)
+#'
+#' @export
+expect_equal_to_reference <- function(current, file, ...){
+  eetr(current=current, file=file, type="equal", ...)
+}
+
+#' @rdname expect_equal_to_reference
+#' @export
+expect_equivalent_to_reference <- function(current, file, ...){
+  eetr(current=current, file=file, type="equivalent", ...)
+}
+
+eetr <- function (current, file, type=c("equal","equivalent"), ...){
+
+    if (file.exists(file)){
+        out <- if (type=="equal")
+                  tinytest::expect_equal(current, readRDS(file), ...)
+                else
+                  tinytest::expect_equivalent(current, readRDS(file), ...)
+        if (!out){
+           diff <- attr(out, "diff")
+           diff <- paste(
+                    sprintf("current does not match target read from %s\n", file)
+                    , diff)
+           attr(out,"diff") <- diff
+        }
+        out
+    } else {
+        tinytest::expect_null(saveRDS(current, file)
+                , info=sprintf("Stored value in %s", file))
+    }
+}
+
+
 
 #' Report side effects for expressions in test files
 #'
@@ -538,7 +757,7 @@ expect_message <- function(current, pattern=".*", info=NA_character_){
 #' report_side_effects(FALSE)
 #'
 #' # only report changes in environment variables
-#' report_side_effects(pwd=FALSE)
+#' report_side_effects(report=FALSE, envvar=TRUE)
 #'
 #' @export
 report_side_effects <- function(report=TRUE, envvar=report, pwd=report, files=report){
