@@ -2,6 +2,46 @@
 #' @importFrom parallel makeCluster parLapply stopCluster
 {}
 
+# directory from which run_test_file() was called (i.e. before it temporarily
+# changes directory
+
+call_wd <- (function(){
+  CALLDIR <- ""
+  function(dir=NULL){
+    if (is.null(dir)){
+      return(CALLDIR)
+    } else {
+      # only set when not set previously
+      if (CALLDIR == "" || dir == "") CALLDIR <<- dir
+    }
+    CALLDIR
+  }
+})()
+
+
+set_call_wd <- function(dir){
+  call_wd(dir)
+}
+
+#' Get workding dir from where a test was initiated
+#'
+#' A test runner, like \code{\link{run_test_file}} changes
+#' R's working directory to the location of the test file temporarily 
+#' while the tests run. This function can be used from within the
+#' test file to get R's working directory at the time \code{run_test_file} 
+#' (or one of it's siblings)
+#' was called.
+#'
+#'
+#' @return \code{[character]} A path.
+#' @examples
+#' get_call_wd()
+#' @export
+get_call_wd <- function(){
+  call_wd()
+}
+
+
 
 # reference object to store or ignore output
 # of 'expect' functions
@@ -48,17 +88,9 @@ output <- function(){
   e$file
   
   # will be set by exit_file()
-  e$exit <- FALSE
-  e$exitmsg <- ""
-  e$exit_msg <- function(print){
-    if(print){ 
-      plural <- e$lst != e$fst
-      if (plural) catf("\nExited '%s' at lines %d-%d. %s"
-                     , basename(e$file), e$fst, e$lst, e$exitmsg)
-      else catf("\nExited '%s' at line %d. %s"
-              , basename(e$file), e$fst, e$exitmsg)
-    }
-  }
+  e$exit     <- FALSE
+  e$exitmsg  <- ""
+  e$exit_msg <- function() sprintf("[Exited at #%d: %s]", e$fst, e$exitmsg)              
   
   e
 }
@@ -221,12 +253,16 @@ add_locally_masked_functions <- function(envir, output){
   envir$expect_equivalent   <- capture(expect_equivalent, output)
   envir$expect_true         <- capture(expect_true, output)
   envir$expect_false        <- capture(expect_false, output)
+  envir$expect_inherits     <- capture(expect_inherits, output)
   envir$expect_null         <- capture(expect_null, output)
   envir$expect_message      <- capture(expect_message, output)
   envir$expect_warning      <- capture(expect_warning, output)
   envir$expect_error        <- capture(expect_error, output)
+  envir$expect_stdout       <- capture(expect_stdout, output)
   envir$expect_identical    <- capture(expect_identical, output)
   envir$expect_silent       <- capture(expect_silent, output)
+  envir$expect_equal_to_reference      <- capture(expect_equal_to_reference, output)
+  envir$expect_equivalent_to_reference <- capture(expect_equivalent_to_reference, output)
   envir$exit_file           <- capture_exit(exit_file, output)
   envir$ignore              <- ignore
   envir$at_home             <- tinytest::at_home
@@ -249,7 +285,7 @@ add_locally_masked_functions <- function(envir, output){
 #'
 #' 
 #' @param package the name of the extension package, given as name or character string.
-#' @param quietly Passed to \code{\link[base]{require}}.
+#' @param quietly Passed to \code{\link{require}}.
 #'
 #' @return A named \code{list}, with the package name and the names of the
 #'   functions registered by \code{package} to extend \pkg{tinytest}. A message
@@ -319,24 +355,26 @@ capture_using <- function(fun, envir, output){
 #'
 #' @section The tinytest API:
 #'
-#' Packages can extend \pkg{tinytest} with expectation functions \emph{if and only}
-#' if the following requirements are satisfied.
+#' Packages can extend \pkg{tinytest} with expectation functions \emph{if and only
+#' if} the following requirements are satisfied.
 #'
 #' \enumerate{
 #'  \item{The extending functions return a \code{\link{tinytest}} object.  This 
-#'        can be created by calling \code{tinytest()} with the arguments
+#'        can be created by calling \code{tinytest()} with the arguments (defaults,
+#'        if any, are in brackets):
 #'    \itemize{
 #'      \item{\code{result}: A \code{logical} scalar: \code{TRUE} or \code{FALSE} (not
 #'            \code{NA}) }
 #'      \item{\code{call}: The \code{call} to the expectation function. Usually the 
 #'            result of \code{sys.call(sys.parent(1))} }
-#'      \item{\code{diff}: A \code{character} scalar, with a long description of the 
+#'      \item{\code{diff} (\code{NA_character_}): A \code{character} scalar, with a long description of the 
 #'            difference. Sentences may be separated by \code{"\\n"}.}
-#'      \item{\code{short}: Either \code{"data"}, if the difference is in the 
+#'      \item{\code{short} (\code{NA_character_}): Either \code{"data"}, if the difference is in the 
 #'            data. \code{"attr"} when attributes differ or \code{"xcpt"} when 
 #'            an expectation about an exception is not met. If there are 
 #'            differences in data and in attributes, the attributes take 
 #'            precedence.}
+#'      \item{\code{info}} (\code{NA_character_}): A user-defined message.
 #'    }
 #'    Observe that this requires the extending package to add \pkg{tinytest} to 
 #'    the \code{Imports} field in the package's \code{DESCRIPTION} file (this 
@@ -352,6 +390,8 @@ capture_using <- function(fun, envir, output){
 #'         functions start with \code{expect_}.}
 #'   \item{Explain to users of the extension package how to use the extension 
 #'         (see \code{\link{using}}).}
+#'   \item{include an \code{info} argument to \code{expect_} functions that 
+#'    is passed to \code{tinytest()}}.
 #' }
 #'
 #'
@@ -384,12 +424,15 @@ register_tinytest_extension <- function(pkg, functions){
 #' @param file \code{[character]} File location of a .R file.
 #' @param at_home \code{[logical]} toggle local tests.
 #' @param verbose \code{[integer]} verbosity level. 0: be quiet, 1: print
-#'   status per file, 2: print status per test expression.
+#'   status per file, 2: print status and increase counter after each test expression.
 #' @param color \code{[logical]} toggle colorize counts in verbose mode (see Note)
 #' @param remove_side_effects \code{[logical]} toggle remove user-defined side
 #'   effects? See section on side effects.
 #' @param side_effects \code{[logical|list]} Either a logical,
 #' or a list of arguments to pass to \code{\link{report_side_effects}}.
+#' @param set_env \code{[named list]}. Key=value pairs of environment variables
+#' that will be set before the test file is run and reset afterwards. These are not
+#' counted as side effects of the code under scrutiny.
 #' @param ... Currently unused
 #' 
 #' @details
@@ -402,31 +445,30 @@ register_tinytest_extension <- function(pkg, functions){
 #' The graphics device is set to \code{pdf(file=tempfile())} for the run of the
 #' test file.
 #'
-#' @section User-defined side effects:
+#' @section Side-effects caused by test code:
 #' 
-#' All calls to \code{\link[base]{Sys.setenv}} and \code{\link[base]{options}}
+#' All calls to \code{\link{Sys.setenv}} and \code{\link{options}}
 #' defined in a test file are captured and undone once the test file has run,
 #' if \code{remove_side_effects} is set to \code{TRUE}.
 #' 
 #' @section Tracking side effects:
 #'
-#' Certain side effects can be tracked, even when they are not explicitly evoked
-#' in the test file. See \code{\link{report_side_effects}} for side effects tracked
-#' by \pkg{tinytest}.
-#' Calls to \code{report_side_effects} within the test file overrule
-#' settings provided with this function.
+#' Certain side effects can be tracked, even when they are not explicitly
+#' evoked in the test file. See \code{\link{report_side_effects}} for side
+#' effects tracked by \pkg{tinytest}.  Calls to \code{report_side_effects}
+#' within the test file overrule settings provided with this function.
 #'
 #' 
 #'
 #' @note
 #' Not all terminals support ansi escape characters, so colorized output can be
-#' switched off. This can also be done globally by setting \code{options(tt.pr.color=FALSE)}.
-#' Some terminals that do support ansi escape characters may contain
-#' bugs. An example is the RStudio terminal (RStudio 1.1) running on Ubuntu 16.04
-#' (and possibly other OSs).
+#' switched off. This can also be done globally by setting
+#' \code{options(tt.pr.color=FALSE)}.  Some terminals that do support ansi
+#' escape characters may contain bugs. An example is the RStudio terminal
+#' (RStudio 1.1) running on Ubuntu 16.04 (and possibly other OSs).
 #'
-#' @return   A \code{list} of class \code{tinytests}, which is a list
-#'    of \code{\link{tinytest}} objects.
+#' @return A \code{list} of class \code{tinytests}, which is a list of
+#' \code{\link{tinytest}} objects.
 #'
 #' @examples
 #' # create a test file, in temp directory
@@ -465,15 +507,25 @@ run_test_file <- function( file
                          , color   = getOption("tt.pr.color", TRUE)
                          , remove_side_effects = TRUE 
                          , side_effects = FALSE
+                         , set_env = list()
                          , ...){
 
   if (!file_test("-f", file)){
     stop(sprintf("'%s' does not exist or is a directory",file),call.=FALSE)
   }
 
+  t0 <- Sys.time()
+
+  # set environment variables (if any) to control the R environment during testing.
+  if (length(set_env) > 0){
+    # first, record current settings
+    old_env_var <- sapply(names(set_env), Sys.getenv, USE.NAMES=TRUE)
+    # new settings
+    do.call(Sys.setenv, set_env)
+  }
   ## where to come back after running the file
   oldwd <- getwd()
-
+  set_call_wd(oldwd)
 
 
   # make sure that plots get redirected to oblivion
@@ -492,6 +544,7 @@ run_test_file <- function( file
       ## Clean up tinytest side effects
       # go back to the original working directory
       setwd(oldwd)
+      set_call_wd("")
       # unset 'at_home' marker
       Sys.unsetenv("TT_AT_HOME")
       if ( remove_side_effects ){ ## Clean up user side effects
@@ -501,6 +554,8 @@ run_test_file <- function( file
         reset_options(oldop)
       }
       grDevices::dev.off()
+      # return env var to values before running run_test_file
+      sapply(names(set_env), function(x) Sys.setenv(x = old_env_var[x]))
   })
 
 
@@ -542,6 +597,7 @@ run_test_file <- function( file
   local_report_files  <- capture(report_files, o)
 
   # parse file, store source reference.
+  check_double_colon(filename=file)
   parsed <- parse(file=file, keep.source=TRUE)
   src <- attr(parsed, "srcref")
   o$file <- file
@@ -559,27 +615,82 @@ run_test_file <- function( file
     o$fst  <- src[[i]][1]
     o$lst  <- src[[i]][3]
     o$call <- expr
-    if ( !o$exit ) eval(expr, envir=e)
-    else {
-      o$exit_msg(verbose >= 1)
-      break
-    }
+
+    if ( !o$exit ) eval(expr, envir=e) else break
+
     local_report_envvar(sidefx)
     local_report_cwd(sidefx)
     local_report_files(sidefx)
-    if (verbose == 2) print_status(prfile, o, color)
+    if (verbose == 2) print_status(prfile, o, color, print=TRUE)
+  }
+  td <- abs(Sys.time() - t0)
+  tx <- humanize(td, color=color)
+  if (verbose == 1){ 
+    # always when run in parallel. And we can only print once in that case
+    str <- print_status(prfile, o, color, print=FALSE)
+    if (o$exit) catf("%s %s %s\n", str, tx, o$exit_msg())
+    else catf("%s %s\n", str, tx)
+  }
+  if (verbose >= 2){ 
+    str <- if (o$exit) catf("%s %s\n", tx, o$exit_msg())
+           else catf("%s\n", tx)
   }
-  if (verbose == 1) print_status(prfile, o, color)
-  if (verbose >= 1) catf("\n")
- 
 
   # returns a 'list' of 'tinytest' objects
   test_output <- o$gimme()
-  structure(test_output, class="tinytests")
+  structure(test_output, class="tinytests", duration=td)
+}
+
+# readable output from a number of seconds.
+humanize <- function(x, color=TRUE){
+  x <- as.numeric(x)
+  # ms units
+  str <-  if (x < 0.1){ 
+            trimws(sprintf("%4.0fms",1000*x))
+          } else if (x < 60 ){ 
+            trimws(sprintf("%3.1fs",x)) 
+          } else if (x < 3600){
+            m <- x %/% 60
+            s <- x - m*60
+            trimws(sprintf("%2.0fm %3.1fs", m, s))
+          } else {
+            # fall-through: hours, minutes, seconds.
+            h <- x %/% 3600
+            m <- (x - 3600 * h)%/% 60
+            s <- x - 3600 * h - 60*m
+            sprintf("%dh %dm %3.1fs", h,m,s)
+          }
+  col <- if (x<0.1) "cyan" else "blue"
+  if (color) color_str(str, col) else str
+}
+
+color_str <- function(x, color){
+  cmap <- c(cyan=36, red=31, green=32, blue = 34)
+  sprintf("\033[0;%dm%s\033[0m", cmap[color], x)
 }
 
 
-print_status <- function(filename, env, color){
+check_double_colon <- function(filename){
+  txt <- readLines(filename, warn=FALSE)
+  i <- grepl("tinytest::expect", txt) & !grepl("#.*tinytest::expect", txt) 
+  if (!any(i)) return(NULL)
+
+  line_numbers <- which(i)
+  occurrences  <- sub("^.*tinytest::expect","tinytest::expect",txt[i])
+  occurrences  <- sub("\\(.*","",occurrences)
+
+  prefix <- 
+" You are using 'tinytest::' to express test expectations.
+ The results from these tests are not collected. Found the following occurrences:
+"
+ issues  <- sprintf("> %s#%03d: %s",basename(filename),line_numbers,occurrences)
+ issues  <- paste(issues, collapse="\n ")
+ postfix <- "\n Remove the 'tinytest::' prefix to register the test results." 
+ message(paste(prefix, issues, postfix), call.=FALSE)
+
+}
+
+print_status <- function(filename, env, color, print=TRUE){
   prefix <- sprintf("\r%s %4d tests", filename, env$ntest())
   # print status after counter
   fails <- if ( env$ntest() == 0 ) "  " # print nothing if nothing was tested
@@ -589,7 +700,8 @@ print_status <- function(filename, env, color){
   side <- if (env$nside() == 0) ""
   else  sprintf(if (color) "\033[0;93m%d side-effects\033[0m" else "%d side-effects", env$nside())  
 
-  cat(prefix, fails, side, sep=" ")
+  if(print) cat(prefix, fails, side, sep=" ")
+  else paste(prefix, fails, side, sep=" ")
 }
 
 
@@ -609,7 +721,7 @@ print_status <- function(filename, env, color){
 #' @param remove_side_effects \code{[logical]} toggle remove user-defined side 
 #'  effects. Environment variables (\code{Sys.setenv()}) and options (\code{options()})
 #'  defined in a test file are reset before running the next test file (see details).
-#' @param cluster A \code{\link[parallel]{makeCluster}} object. 
+#' @param cluster A \code{\link{makeCluster}} object. 
 #' @param lc_collate \code{[character]} Locale setting used to sort the
 #'  test files into the order of execution. The default \code{NA} ensures
 #'  current locale is used. Set this e.g. to \code{"C"} to ensure bytewise
@@ -623,11 +735,11 @@ print_status <- function(filename, env, color){
 #' determines how alphabets are sorted).  For this reason it is a good idea to
 #' create test files that run independent of each other so their order of
 #' execution does not matter. In tinytest, test files cannot share variables.
-#' The default behavior of test runners furher discourages interdependence by
+#' The default behavior of test runners further discourages interdependence by
 #' resetting environment variables and options that are set in a test file
 #' after the file is executed. If an environment variable needs to survive a
 #' single file, use \code{base::Sys.setenv()} explicitly.  Similarly, if an
-#' option setting needs to survive, use \code{base::options}
+#' option setting needs to survive, use \code{base::options()}
 #'
 #' @section Parallel tests:
 #'
@@ -661,8 +773,8 @@ print_status <- function(filename, env, color){
 #' dat <- as.data.frame(out)
 #'
 #' @family test-files
-#' @seealso \code{\link[parallel]{makeCluster}},
-#' \code{\link[parallel]{clusterEvalQ}}, \code{\link[parallel]{clusterExport}}
+#' @seealso \code{\link{makeCluster}},
+#' \code{\link{clusterEvalQ}}, \code{\link{clusterExport}}
 #'
 #' @export
 run_test_dir <- function(dir="inst/tinytest", pattern="^test.*\\.[rR]$"
@@ -674,6 +786,7 @@ run_test_dir <- function(dir="inst/tinytest", pattern="^test.*\\.[rR]$"
                        , lc_collate = getOption("tt.collate",NA)
                        , ... ){
 
+  t0 <- Sys.time()
 
   testfiles <- dir(dir, pattern=pattern, full.names=TRUE)
   testfiles <- locale_sort(testfiles, lc_collate=lc_collate)
@@ -683,7 +796,8 @@ run_test_dir <- function(dir="inst/tinytest", pattern="^test.*\\.[rR]$"
   if ( !inherits(cluster, "cluster") ){
     # set pwd here, to save time in run_test_file.
     oldwd <- getwd()
-    on.exit(setwd(oldwd))
+    set_call_wd(oldwd)
+    on.exit({setwd(oldwd); set_call_wd("")})
     setwd(dir)  
     test_output <- lapply(basename(testfiles), run_test_file
                            , at_home = at_home
@@ -697,9 +811,11 @@ run_test_dir <- function(dir="inst/tinytest", pattern="^test.*\\.[rR]$"
         , run_test_file, at_home = at_home, verbose = min(verbose,1)
         , color = color, remove_side_effects = TRUE, ...)
   }
+
+  td <- abs(Sys.time() - t0)
   # by using '(parL)|(l)apply' we get a list of tinytests objects. We need to unwind
   # one level to a list of 'tinytest' objects and class it 'tinytests'.
-  structure(unlist(test_output,recursive=FALSE), class="tinytests")
+  structure(unlist(test_output,recursive=FALSE), class="tinytests", duration=td)
 }
 
 
@@ -773,20 +889,30 @@ at_home <- function(){
 #' used by \code{R CMD check} or by a user that installed a package that
 #' uses the \pkg{tinytest} test infrastructure.
 #'
-#' @param pkgname \code{[character]} scalar. Name of the package
-#' @param testdir \code{[character]} scalar. Path to installed directory, relative
-#' to the working directory of \code{R CMD check}.
+#' @param pkgname \code{[character]} scalar. Name of the package, as in the \code{DESCRIPTION} file.
+#' @param testdir \code{[character]} scalar. Path to installed directory. By default 
+#'        tinytest assumes that test files are in \code{inst/tinytest/}, which means
+#'        that after installation and thus during \code{R CMD check} they are in 
+#'        \code{tinytest/}. See details for using alternate paths.
+#' @param lib.loc \code{[character]} scalar. location where the package is installed.
 #' @param at_home \code{[logical]} scalar. Are we at home? (see Details)
-#' @param ncpu A positive integer, or a \code{\link[parallel]{makeCluster}} object.
+#' @param ncpu A positive integer, or a \code{\link{makeCluster}} object.
 #' @param ... extra arguments passed to \code{\link{run_test_dir}} (e.g. \code{ncpu}).
 #'
 #'
 #' @section Details:
-#' We set \code{at_home=FALSE} by default so \code{R CMD check} will run the same
-#' as at CRAN. See the package vignette (Section 4) for tips on how to set up
-#' the package structure.
+#' We set \code{at_home=FALSE} by default so \code{R CMD check} will run the
+#' same as at CRAN. See the package vignette (Section 4) for tips on how to set
+#' up the package structure.
 #' \code{vignette("using_tinytest",package="tinytest")}.
 #'
+#' Package authors who want to avoid installing tests with the package can
+#' create a directory under \code{tests}. If the test directoy is called
+#' \code{"tests/foo"}, use \code{test_package("pkgname", testdir="foo")} in
+#' \code{tests/tinytest.R}.
+#'
+#' 
+#'
 #' @return If \code{interactive()}, a \code{tinytests} object. If not
 #'  \code{interactive()}, an error is thrown when at least one test fails.
 #'
@@ -800,15 +926,33 @@ at_home <- function(){
 #'     tinytest::test_package("your package name")
 #' }
 #' @export
-test_package <- function(pkgname, testdir = "tinytest"
+test_package <- function(pkgname, testdir = "tinytest", lib.loc=NULL
                        , at_home=FALSE, ncpu=NULL, ...){
+  oldlibpaths <- .libPaths()
+
+  if (!is.null(lib.loc)){ 
+    e <- new.env()
+    e$libs <- c(lib.loc, oldlibpaths)
+      
+    if (!dir.exists(lib.loc)) 
+      warnf("lib.loc '%s' not found.", lib.loc)
+    .libPaths(c(lib.loc, oldlibpaths))
+  }
+
   on.exit({
     if ( is.numeric(ncpu) ) parallel::stopCluster(cluster)
+    .libPaths(oldlibpaths)
   })
 
-  testdir <- system.file(testdir, package=pkgname)
-  if ( testdir == "" ){
-    stopf("testdir '%s' not found for package '%s'",testdir, pkgname)
+  if (!dir.exists(testdir)){ # if not customized test dir
+    # find the installed test dir
+    new_testdir <- system.file(testdir, package=pkgname, lib.loc=lib.loc)
+    if (new_testdir == ""){
+      stopf("testdir '%s' not found for package '%s'",testdir, pkgname)
+    } else {
+      testdir <- new_testdir
+    }
+
   }
 
   # set up cluster if required
@@ -819,20 +963,25 @@ test_package <- function(pkgname, testdir = "tinytest"
 
   # By now we have a cluster, or NULL. Load the pkg under scrutiny.
   if ( is.null(cluster) ){
-    library(pkgname, character.only=TRUE)
+    library(pkgname, character.only=TRUE, lib.loc=lib.loc)
   } else {
-    parallel::clusterCall(cluster, library, pkgname, character.only=TRUE)
+    if (!is.null(lib.loc)){
+      # to prevent a R CMD check warning we must create a dummy libs here
+      # as well
+      libs <- NULL
+      parallel::clusterExport(cluster, "libs", envir = e)
+      parallel::clusterEvalQ(cluster, .libPaths(libs))
+    }
+    parallel::clusterCall(cluster, library, pkgname, character.only=TRUE, lib.loc=lib.loc)
   }
 
   out <- run_test_dir(testdir, at_home=at_home, cluster=cluster,...) 
   i_fail <- sapply(out, isFALSE)
-  if ( any(i_fail) ){
-    msg <- paste( sapply(out[i_fail], format.tinytest, type="long"), collapse="\n")
-    msg <- paste(msg, "\n")
-    if (!interactive()) stop(msg, call.=FALSE)
-  } else {
-    out
-  }
+  if ( any(i_fail) && !interactive() ){
+    writeLines(vapply(out[i_fail], format.tinytest, "", type="long"))
+    stop(sum(i_fail), " out of ", length(out), " tests failed", call.=FALSE)
+  } 
+  out
 }
 
 
@@ -850,6 +999,9 @@ test_package <- function(pkgname, testdir = "tinytest"
 #' @param pkgdir \code{[character]} Package directory
 #' @param testdir \code{[character]} Name of directory under \code{pkgdir/inst}
 #'   containing test files.
+#' @param pattern \code{[character]} A regular expression that is used to find
+#'   scripts in \code{dir} containing tests (by default \code{.R} or \code{.r}
+#'   files starting with \code{test}).
 #' @param at_home \code{[logical]} toggle local tests.
 #' @param ncpu \code{[numeric]} number of CPUs to use during the testing phase.
 #' @param verbose \code{[logical]} toggle verbosity during execution
@@ -857,6 +1009,10 @@ test_package <- function(pkgname, testdir = "tinytest"
 #'   effects? See section on side effects.
 #' @param side_effects \code{[logical|list]} Either a logical,
 #' or a list of arguments to pass to \code{\link{report_side_effects}}.
+#' @param lc_collate \code{[character]} Locale setting used to sort the
+#'  test files into the order of execution. The default \code{NA} ensures
+#'  current locale is used. Set this e.g. to \code{"C"} to ensure bytewise
+#'  and more platform-independent sorting (see details in \code{\link{run_test_dir}}.
 #' @param keep_tempdir \code{[logical]} keep directory where the pkg is
 #'   installed and where tests are run? If \code{TRUE}, the directory is not
 #'   deleted and it's location is printed.
@@ -872,11 +1028,13 @@ test_package <- function(pkgname, testdir = "tinytest"
 #' @family test-files
 #' @export
 build_install_test <- function(pkgdir="./", testdir="tinytest"
+                             , pattern="^test.*\\.[rR]$"
                              , at_home=TRUE
                              , verbose=getOption("tt.verbose",2)
                              , ncpu = 1
                              , remove_side_effects=TRUE
                              , side_effects=FALSE
+                             , lc_collate = getOption("tt.collate",NA)
                              , keep_tempdir=FALSE){
   oldwd <- getwd()
   tdir  <- tempfile()
@@ -892,6 +1050,8 @@ build_install_test <- function(pkgdir="./", testdir="tinytest"
 
   pkgname <- read.dcf(file.path(pkg, "DESCRIPTION"))[1]
 
+  pattern <- gsub("\\", "\\\\", pattern, fixed=TRUE)
+
   dir.create(tdir)
   setwd(tdir)
 
@@ -908,6 +1068,7 @@ build_install_test <- function(pkgdir="./", testdir="tinytest"
   script <- "
 suppressPackageStartupMessages({
   pkgname <- '%s'
+  pattern <- '%s'
   tdir    <- '%s'
   testdir <- '%s'
   at_home <- %s
@@ -915,6 +1076,7 @@ suppressPackageStartupMessages({
   remove_side_effects <- %s
   side_effects <- %s
   ncpu    <- %d
+  lc_collate <- %s
 
   #        pkgname       tdir
   library(pkgname, lib.loc=tdir,character.only=TRUE)
@@ -930,10 +1092,12 @@ if (ncpu > 1){
 #                                testdir       pkgname       tdir
 out <- run_test_dir(system.file(testdir, package=pkgname, lib.loc=tdir)
                    , at_home=at_home
+                   , pattern=pattern
                    , verbose=verbose
                    , remove_side_effects=remove_side_effects
                    , side_effects=side_effects
-                   , cluster=cluster)
+                   , cluster=cluster
+                   , lc_collate=lc_collate)
 
 saveRDS(out, file='output.RDS')
 
@@ -941,13 +1105,15 @@ if (!is.null(cluster)) parallel::stopCluster(cluster)
 "
   scr <- sprintf(script
         , pkgname
+        , pattern
         , normalizePath(tdir, winslash="/", mustWork=FALSE)
         , testdir
         , at_home
         , verbose
         , remove_side_effects
         , side_effects
-        , ncpu)
+        , ncpu
+        , lc_collate)
 
   write(scr, file="test.R")
   system("Rscript test.R")
